#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import shutil

import click

from polyaxon import settings
from polyaxon.client.transport import Transport
from polyaxon.deploy.operators.compose import ComposeOperator
from polyaxon.deploy.operators.docker import DockerOperator
from polyaxon.deploy.operators.helm import HelmOperator
from polyaxon.deploy.operators.kubectl import KubectlOperator
from polyaxon.deploy.schemas.deployment import DeploymentConfig
from polyaxon.deploy.schemas.deployment_types import DeploymentCharts, DeploymentTypes
from polyaxon.exceptions import PolyaxonException, PolyaxonOperatorException
from polyaxon.managers.compose import ComposeConfigManager
from polyaxon.utils.formatting import Printer


class DeployConfigManager:
    def __init__(
        self,
        config: DeploymentConfig = None,
        filepath=None,
        deployment_type=False,
        manager_path=None,
        dry_run=False,
    ):
        self.config = config
        self.filepath = filepath
        self.type = deployment_type
        self.manager_path = manager_path
        self.dry_run = dry_run
        self.kubectl = KubectlOperator()
        self.helm = HelmOperator()
        self.docker = DockerOperator()
        self.compose = ComposeOperator()

    @property
    def deployment_type(self):
        if self.type:
            return self.type
        if self.config and self.config.deployment_type:
            return self.config.deployment_type
        return DeploymentTypes.KUBERNETES

    @property
    def deployment_version(self):
        if self.config and self.config.deployment_version:
            return self.config.deployment_version
        return None

    @property
    def deployment_namespace(self):
        if self.config and self.config.namespace:
            return self.config.namespace
        return "polyaxon"

    @property
    def k8s_chart(self):
        deployment_chart = DeploymentCharts.PLATFORM
        if self.config and self.config.deployment_chart:
            deployment_chart = self.config.deployment_chart
        if deployment_chart == DeploymentCharts.PLATFORM:
            return "polyaxon/polyaxon"
        else:
            return "polyaxon/agent"

    @property
    def release_name(self):
        if self.config and self.config.release_name:
            return self.config.release_name
        deployment_chart = DeploymentCharts.PLATFORM
        if self.config and self.config.deployment_chart:
            deployment_chart = self.config.deployment_chart
        if deployment_chart == DeploymentCharts.PLATFORM:
            return "polyaxon"
        else:
            return "agent"

    @property
    def is_kubernetes(self):
        return self.deployment_type in {
            DeploymentTypes.KUBERNETES,
            DeploymentTypes.MINIKUBE,
            DeploymentTypes.MICRO_K8S,
        }

    @property
    def is_docker_compose(self):
        return self.deployment_type == DeploymentTypes.DOCKER_COMPOSE

    @property
    def is_docker(self):
        return self.deployment_type == DeploymentTypes.DOCKER

    @property
    def is_heroku(self):
        return self.deployment_type == DeploymentTypes.HEROKU

    @property
    def is_valid(self):
        return self.deployment_type in DeploymentTypes.VALUES

    def check_for_kubernetes(self):
        # Deployment on k8s requires helm & kubectl to be installed
        if not self.kubectl.check():
            raise PolyaxonException("kubectl is required to run this command.")
        Printer.print_success("kubectl is installed")

        if not self.helm.check():
            raise PolyaxonException("helm is required to run this command.")
        Printer.print_success("helm is installed")

        # Check that polyaxon/polyaxon is set and up-to date
        self.helm.execute(
            args=["repo", "add", "polyaxon", "https://charts.polyaxon.com"]
        )
        self.helm.execute(args=["repo", "update"])
        return True

    def check_for_docker_compose(self):
        # Deployment on docker compose requires Docker & Docker Compose to be installed
        if not self.docker.check():
            raise PolyaxonException("Docker is required to run this command.")
        Printer.print_success("Docker is installed")

        if not self.compose.check():
            raise PolyaxonException("Docker Compose is required to run this command.")
        Printer.print_success("Docker Compose is installed")

        # Check that .polyaxon/.compose is set and up-to date
        if ComposeConfigManager.is_initialized():
            Printer.print_success("Docker Compose deployment is initialised.")
        return True

    def check_for_docker(self):
        if not self.docker.check():
            raise PolyaxonException("Docker is required to run this command.")
        return True

    def check_for_heroku(self):
        return True

    def nvidia_device_plugin(self):
        return "https://github.com/NVIDIA/k8s-device-plugin/blob/v1.10/nvidia-device-plugin.yml"

    def check(self):
        """Add platform specific checks"""
        if not self.is_valid:
            raise PolyaxonException(
                "Deployment type `{}` not supported".format(self.deployment_type)
            )
        check = False
        if self.is_kubernetes:
            check = self.check_for_kubernetes()
        elif self.is_docker_compose:
            check = self.check_for_docker_compose()
        elif self.is_docker:
            check = self.check_for_docker()
        elif self.is_heroku:
            check = self.check_for_heroku()
        if not check:
            raise PolyaxonException(
                "Deployment `{}` is not valid".format(self.deployment_type)
            )

    def _get_or_create_namespace(self):
        click.echo("Checking {} namespace ...".format(self.deployment_namespace))
        try:
            stdout = self.kubectl.execute(
                args=["get", "namespace", self.deployment_namespace],
                is_json=True,
                stream=settings.CLIENT_CONFIG.debug,
            )
        except PolyaxonOperatorException:
            stdout = None

        if stdout:
            return
        # Create a namespace
        click.echo("Creating {} namespace ...".format(self.deployment_namespace))
        stdout = self.kubectl.execute(
            args=["create", "namespace", self.deployment_namespace],
            is_json=False,
            stream=settings.CLIENT_CONFIG.debug,
        )
        click.echo(stdout)

    def install_on_kubernetes(self):
        self._get_or_create_namespace()

        args = ["install", self.release_name]
        if self.manager_path:
            args += [self.manager_path]
        else:
            args += [self.k8s_chart]

        args += ["--namespace={}".format(self.deployment_namespace)]
        if self.filepath:
            args += ["-f", self.filepath]
        if self.type in [DeploymentTypes.MICRO_K8S, DeploymentTypes.MINIKUBE]:
            args += [
                "--set",
                "gateway.service.type=NodePort,deploymentType={}".format(self.type),
            ]
        if self.deployment_version:
            args += ["--version", self.deployment_version]
        if self.dry_run:
            args += ["--debug", "--dry-run"]

        click.echo("Running install command ...")
        stdout = self.helm.execute(args=args, stream=settings.CLIENT_CONFIG.debug)
        click.echo(stdout)
        Printer.print_success("Deployment finished.")

    def install_on_docker_compose(self):
        path = ComposeConfigManager.get_config_filepath()
        path = "/".join(path.split("/")[:-1])
        # Fetch docker-compose
        Transport().download(
            url="https://github.com/polyaxon/polyaxon-compose/archive/master.tar.gz",
            filename=path + "/file",
            untar=True,
            delete_tar=True,
            extract_path=path,
        )
        # Move necessary info
        shutil.copy(
            path + "/polyaxon-compose-master/docker-compose.yml",
            path + "/docker-compose.yml",
        )
        shutil.copy(
            path + "/polyaxon-compose-master/components.env", path + "/components.env"
        )
        shutil.copy(path + "/polyaxon-compose-master/base.env", path + "/base.env")
        shutil.rmtree(path + "/polyaxon-compose-master/")
        # Generate env from config
        ComposeConfigManager.set_config(self.compose.generate_env(self.config))
        Printer.print_success("Docker Compose deployment is initialised.")
        if self.dry_run:
            Printer.print_success("Polyaxon generated deployment env.")
            return
        self.docker.execute(["volume", "create", "--name=polyaxon-postgres"])
        Printer.print_success("Docker volume created.")
        self.compose.execute(["-f", path + "/docker-compose.yml", "up", "-d"])
        Printer.print_success("Deployment is running in the background.")
        Printer.print_success(
            "You can configure your CLI by running: "
            "polyaxon config set --host=localhost."
        )

    def install_on_docker(self):
        pass

    def install_on_heroku(self):
        pass

    def install(self):
        """Install polyaxon using the current config to the correct platform."""
        if not self.is_valid:
            raise PolyaxonException(
                "Deployment type `{}` not supported".format(self.deployment_type)
            )

        if self.is_kubernetes:
            self.install_on_kubernetes()
        elif self.is_docker_compose:
            self.install_on_docker_compose()
        elif self.is_docker:
            self.install_on_docker()
        elif self.is_heroku:
            self.install_on_heroku()

    def upgrade_on_kubernetes(self):
        args = ["upgrade", self.release_name]
        if self.manager_path:
            args += [self.manager_path]
        else:
            args += [self.k8s_chart]
        if self.filepath:
            args += ["-f", self.filepath]
        if self.type and self.type in [
            DeploymentTypes.MICRO_K8S,
            DeploymentTypes.MINIKUBE,
        ]:
            args += [
                "--set",
                "gateway.service.type=NodePort,deploymentType={}".format(self.type),
            ]
        if self.deployment_version:
            args += ["--version", self.deployment_version]
        args += ["--namespace={}".format(self.deployment_namespace)]
        if self.dry_run:
            args += ["--debug", "--dry-run"]
        click.echo("Running upgrade command ...")
        stdout = self.helm.execute(args=args, stream=settings.CLIENT_CONFIG.debug)
        click.echo(stdout)
        Printer.print_success("Deployment upgraded.")

    def upgrade_on_docker_compose(self):
        self.install_on_docker_compose()

    def upgrade_on_docker(self):
        pass

    def upgrade_on_heroku(self):
        pass

    def upgrade(self):
        """Upgrade deployment."""
        if not self.is_valid:
            raise PolyaxonException(
                "Deployment type `{}` not supported".format(self.deployment_type)
            )

        if self.is_kubernetes:
            self.upgrade_on_kubernetes()
        elif self.is_docker_compose:
            self.upgrade_on_docker_compose()
        elif self.is_docker:
            self.upgrade_on_docker()
        elif self.is_heroku:
            self.upgrade_on_heroku()

    def teardown_on_kubernetes(self, hooks):
        args = ["delete", self.release_name]
        if not hooks:
            args += ["--no-hooks"]
        args += ["--namespace={}".format(self.deployment_namespace)]
        click.echo("Running teardown command ...")
        self.helm.execute(args=args)
        Printer.print_success("Deployment successfully deleted.")

    def teardown_on_docker_compose(self):
        path = ComposeConfigManager.get_config_filepath()
        path = "/".join(path.split("/")[:-1])
        self.compose.execute(["-f", path + "/docker-compose.yml", "down"])

    def teardown_on_docker(self, hooks):
        pass

    def teardown_on_heroku(self, hooks):
        pass

    def teardown(self, hooks=True):
        """Teardown Polyaxon."""
        if not self.is_valid:
            raise PolyaxonException(
                "Deployment type `{}` not supported".format(self.deployment_type)
            )

        if self.is_kubernetes:
            self.teardown_on_kubernetes(hooks=hooks)
        elif self.is_docker_compose:
            self.teardown_on_docker_compose()
        elif self.is_docker:
            self.teardown_on_docker(hooks=hooks)
        elif self.is_heroku:
            self.teardown_on_heroku(hooks=hooks)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# pylint:disable=protected-access
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.exceptions import PolyaxonNotificationException
from polyaxon.notifiers.hipchat_webhook import HipChatWebHookNotifier
from tests.test_notifiers.test_webhook_notification import TestWebHookNotification


class TestHipChatWebHookNotifier(TestWebHookNotification):
    webhook = HipChatWebHookNotifier

    def test_attrs(self):
        assert self.webhook.notification_key == V1ConnectionKind.HIPCHAT
        assert self.webhook.name == "HipChat WebHook"

    def test_prepare(self):
        with self.assertRaises(PolyaxonNotificationException):
            self.webhook._prepare(None)
        with self.assertRaises(PolyaxonNotificationException):
            self.webhook._prepare({})

        context = {"message": "message"}
        assert self.webhook._prepare(context) == {
            "message": context.get("message"),
            "message_format": context.get("message_format", "html"),
            "color": context.get("color"),
            "from": "Polyaxon",
            "attach_to": context.get("attach_to"),
            "notify": context.get("notify", False),
            "card": context.get("card"),
        }


del TestWebHookNotification



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from asyncio import Future
from mock import patch

from polyaxon_sdk import V1StatusCondition

from polyaxon import settings
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import V1K8sResourceSchema
from polyaxon.lifecycle import V1Statuses
from polyaxon.schemas.types import V1ConnectionType
from polyaxon.streams.app.main import STREAMS_URL
from tests.test_streams.base import get_streams_client, set_store
from tests.utils import BaseTestCase


class TestNotifyEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.store_root = set_store()
        settings.AGENT_CONFIG.notification_connections = [
            V1ConnectionType(
                name="slack",
                kind=V1ConnectionKind.SLACK,
                secret=V1K8sResourceSchema(name="some"),
            )
        ]

        self.client = get_streams_client()
        self.base_url = STREAMS_URL + "/namespace/owner/project/runs/uuid/notify"

    def test_notify_with_no_data(self):
        response = self.client.post(self.base_url, json={})
        assert response.status_code == 400

    def test_notify_with_no_condition(self):
        data = {"name": "test", "condition": None, "connections": ["test"]}
        response = self.client.post(self.base_url, json=data)
        assert response.status_code == 400

    def test_notify_with_no_notifications(self):
        data = {
            "name": "test",
            "condition": V1StatusCondition(
                type=V1Statuses.FAILED, status=True
            ).to_dict(),
            "connections": None,
        }
        response = self.client.post(self.base_url, json=data)
        assert response.status_code == 400

        data = {
            "name": "test",
            "condition": V1StatusCondition(
                type=V1Statuses.FAILED, status=True
            ).to_dict(),
            "connections": [],
        }
        response = self.client.post(self.base_url, json=data)
        assert response.status_code == 400

    def test_notify_with_no_agent_notification_connections(self):
        settings.AGENT_CONFIG.notification_connections = []
        data = {
            "name": "test",
            "condition": V1StatusCondition(
                type=V1Statuses.FAILED, status=True
            ).to_dict(),
            "connections": ["test1", "test2"],
        }
        response = self.client.post(self.base_url, json=data)
        assert response.status_code == 400

    def test_notify_with_non_recognized_connections(self):
        data = {
            "name": "test",
            "condition": V1StatusCondition(
                type=V1Statuses.FAILED, status=True
            ).to_dict(),
            "connections": ["test1", "test2"],
        }
        with patch(
            "polyaxon.agents.spawners.async_spawner.AsyncK8SManager"
        ) as manager_mock:
            manager_mock.return_value.setup.return_value = Future()
            manager_mock.return_value.setup.return_value.set_result(None)
            response = self.client.post(self.base_url, json=data)
            assert response.status_code == 200



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.polyflow.termination import V1Termination
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.termination_mark
class TestV1Terminations(BaseTestCase):
    def test_termination_config(self):
        config_dict = {}
        config = V1Termination.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add max_retries
        config_dict["maxRetries"] = 4
        config = V1Termination.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add timeout
        config_dict["timeout"] = 4
        config = V1Termination.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())

        # Add ttl
        config_dict["ttl"] = 40
        config = V1Termination.from_dict(config_dict)
        assert_equal_dict(config_dict, config.to_dict())



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk


class V1ConnectionKind(polyaxon_sdk.V1ConnectionKind):
    CHOICES = (
        (
            polyaxon_sdk.V1ConnectionKind.HOST_PATH,
            polyaxon_sdk.V1ConnectionKind.HOST_PATH,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.VOLUME_CLAIM,
            polyaxon_sdk.V1ConnectionKind.VOLUME_CLAIM,
        ),
        (polyaxon_sdk.V1ConnectionKind.GCS, polyaxon_sdk.V1ConnectionKind.GCS),
        (polyaxon_sdk.V1ConnectionKind.S3, polyaxon_sdk.V1ConnectionKind.S3),
        (polyaxon_sdk.V1ConnectionKind.WASB, polyaxon_sdk.V1ConnectionKind.WASB),
        (
            polyaxon_sdk.V1ConnectionKind.REGISTRY,
            polyaxon_sdk.V1ConnectionKind.REGISTRY,
        ),
        (polyaxon_sdk.V1ConnectionKind.GIT, polyaxon_sdk.V1ConnectionKind.GIT),
        (polyaxon_sdk.V1ConnectionKind.AWS, polyaxon_sdk.V1ConnectionKind.AWS),
        (polyaxon_sdk.V1ConnectionKind.GCP, polyaxon_sdk.V1ConnectionKind.GCP),
        (polyaxon_sdk.V1ConnectionKind.AZURE, polyaxon_sdk.V1ConnectionKind.AZURE),
        (polyaxon_sdk.V1ConnectionKind.MYSQL, polyaxon_sdk.V1ConnectionKind.MYSQL),
        (
            polyaxon_sdk.V1ConnectionKind.POSTGRES,
            polyaxon_sdk.V1ConnectionKind.POSTGRES,
        ),
        (polyaxon_sdk.V1ConnectionKind.ORACLE, polyaxon_sdk.V1ConnectionKind.ORACLE),
        (polyaxon_sdk.V1ConnectionKind.VERTICA, polyaxon_sdk.V1ConnectionKind.VERTICA),
        (polyaxon_sdk.V1ConnectionKind.SQLITE, polyaxon_sdk.V1ConnectionKind.SQLITE),
        (polyaxon_sdk.V1ConnectionKind.MSSQL, polyaxon_sdk.V1ConnectionKind.MSSQL),
        (polyaxon_sdk.V1ConnectionKind.REDIS, polyaxon_sdk.V1ConnectionKind.REDIS),
        (polyaxon_sdk.V1ConnectionKind.PRESTO, polyaxon_sdk.V1ConnectionKind.PRESTO),
        (polyaxon_sdk.V1ConnectionKind.MONGO, polyaxon_sdk.V1ConnectionKind.MONGO),
        (
            polyaxon_sdk.V1ConnectionKind.CASSANDRA,
            polyaxon_sdk.V1ConnectionKind.CASSANDRA,
        ),
        (polyaxon_sdk.V1ConnectionKind.FTP, polyaxon_sdk.V1ConnectionKind.FTP),
        (polyaxon_sdk.V1ConnectionKind.GRPC, polyaxon_sdk.V1ConnectionKind.GRPC),
        (polyaxon_sdk.V1ConnectionKind.HDFS, polyaxon_sdk.V1ConnectionKind.HDFS),
        (polyaxon_sdk.V1ConnectionKind.HTTP, polyaxon_sdk.V1ConnectionKind.HTTP),
        (polyaxon_sdk.V1ConnectionKind.PIG_CLI, polyaxon_sdk.V1ConnectionKind.PIG_CLI),
        (
            polyaxon_sdk.V1ConnectionKind.HIVE_CLI,
            polyaxon_sdk.V1ConnectionKind.HIVE_CLI,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.HIVE_METASTORE,
            polyaxon_sdk.V1ConnectionKind.HIVE_METASTORE,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.HIVE_SERVER2,
            polyaxon_sdk.V1ConnectionKind.HIVE_SERVER2,
        ),
        (polyaxon_sdk.V1ConnectionKind.JDBC, polyaxon_sdk.V1ConnectionKind.JDBC),
        (polyaxon_sdk.V1ConnectionKind.JENKINS, polyaxon_sdk.V1ConnectionKind.JENKINS),
        (polyaxon_sdk.V1ConnectionKind.SAMBA, polyaxon_sdk.V1ConnectionKind.SAMBA),
        (
            polyaxon_sdk.V1ConnectionKind.SNOWFLAKE,
            polyaxon_sdk.V1ConnectionKind.SNOWFLAKE,
        ),
        (polyaxon_sdk.V1ConnectionKind.SSH, polyaxon_sdk.V1ConnectionKind.SSH),
        (
            polyaxon_sdk.V1ConnectionKind.CLOUDANT,
            polyaxon_sdk.V1ConnectionKind.CLOUDANT,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.DATABRICKS,
            polyaxon_sdk.V1ConnectionKind.DATABRICKS,
        ),
        (polyaxon_sdk.V1ConnectionKind.SEGMENT, polyaxon_sdk.V1ConnectionKind.SEGMENT),
        (polyaxon_sdk.V1ConnectionKind.SLACK, polyaxon_sdk.V1ConnectionKind.SLACK),
        (polyaxon_sdk.V1ConnectionKind.DISCORD, polyaxon_sdk.V1ConnectionKind.DISCORD),
        (
            polyaxon_sdk.V1ConnectionKind.MATTERMOST,
            polyaxon_sdk.V1ConnectionKind.MATTERMOST,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.PAGERDUTY,
            polyaxon_sdk.V1ConnectionKind.PAGERDUTY,
        ),
        (
            polyaxon_sdk.V1ConnectionKind.HIPCHAT,
            polyaxon_sdk.V1ConnectionKind.HIPCHAT,
        ),
        (polyaxon_sdk.V1ConnectionKind.WEBHOOK, polyaxon_sdk.V1ConnectionKind.WEBHOOK),
        (polyaxon_sdk.V1ConnectionKind.CUSTOM, polyaxon_sdk.V1ConnectionKind.CUSTOM),
    )

    MOUNT_VALUES = {
        polyaxon_sdk.V1ConnectionKind.HOST_PATH,
        polyaxon_sdk.V1ConnectionKind.VOLUME_CLAIM,
    }

    BLOB_VALUES = {
        polyaxon_sdk.V1ConnectionKind.GCS,
        polyaxon_sdk.V1ConnectionKind.S3,
        polyaxon_sdk.V1ConnectionKind.WASB,
    }

    ARTIFACT_VALUES = BLOB_VALUES | MOUNT_VALUES

    HOST_VALUES = {
        polyaxon_sdk.V1ConnectionKind.GIT,
        polyaxon_sdk.V1ConnectionKind.REGISTRY,
    }

    @classmethod
    def is_bucket(cls, kind):
        return kind in cls.BLOB_VALUES

    @classmethod
    def is_mount(cls, kind):
        return kind in cls.MOUNT_VALUES

    @classmethod
    def is_host_path(cls, kind):
        return kind == cls.HOST_PATH

    @classmethod
    def is_volume_claim(cls, kind):
        return kind == cls.VOLUME_CLAIM

    @classmethod
    def is_artifact(cls, kind):
        return kind in cls.ARTIFACT_VALUES

    @classmethod
    def is_git(cls, kind):
        return kind == cls.GIT

    @classmethod
    def is_ssh(cls, kind):
        return kind == cls.SSH

    @classmethod
    def is_registry(cls, kind):
        return kind == cls.REGISTRY

    @classmethod
    def is_s3(cls, kind):
        return kind == cls.S3

    @classmethod
    def is_wasb(cls, kind):
        return kind == cls.WASB

    @classmethod
    def is_gcs(cls, kind):
        return kind == cls.GCS



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click


@click.command()
@click.option(
    "--kind",
    help="The notification kind.",
)
@click.option(
    "--owner",
    help="The project owner.",
)
@click.option(
    "--project",
    help="The project containing the operation.",
)
@click.option("--run-uuid", help="The run uuid.")
@click.option("--run-name", help="The run name.")
@click.option(
    "--condition",
    help="The run condition to notify.",
)
def notify(kind, owner, project, run_uuid, run_name, condition):
    """Notifier command."""
    import ujson

    from polyaxon.lifecycle import V1StatusCondition
    from polyaxon.notifiers import NOTIFIERS, NotificationSpec

    condition = ujson.loads(condition)
    condition = V1StatusCondition.get_condition(**condition)
    notification = NotificationSpec(
        kind=kind,
        owner=owner,
        project=project,
        uuid=run_uuid,
        name=run_name,
        condition=condition,
    )
    NOTIFIERS[kind].execute(notification=notification)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import ValidationError, fields, validate, validates_schema

from polyaxon.deploy.schemas.auth import AuthSchema
from polyaxon.deploy.schemas.deployment_types import DeploymentCharts, DeploymentTypes
from polyaxon.deploy.schemas.email import EmailSchema
from polyaxon.deploy.schemas.ingress import IngressSchema
from polyaxon.deploy.schemas.intervals import IntervalsSchema
from polyaxon.deploy.schemas.operators import OperatorsSchema
from polyaxon.deploy.schemas.rbac import RBACSchema
from polyaxon.deploy.schemas.root_user import RootUserSchema
from polyaxon.deploy.schemas.security_context import SecurityContextSchema
from polyaxon.deploy.schemas.service import (
    AgentServiceSchema,
    ApiServiceSchema,
    ExternalServicesSchema,
    HelperServiceSchema,
    HooksSchema,
    OperatorServiceSchema,
    PostgresqlSchema,
    RabbitmqSchema,
    RedisSchema,
    ServiceSchema,
    WorkerServiceSchema,
)
from polyaxon.deploy.schemas.service_types import ServiceTypes
from polyaxon.deploy.schemas.ssl import SSLSchema
from polyaxon.deploy.schemas.ui import UISchema
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
from polyaxon.schemas.types import ConnectionTypeSchema


def validate_connections(artifacts_store, connections):
    connections = connections or []

    connection_names = set()

    if artifacts_store:
        connection_names.add(artifacts_store.name)

    for c in connections:
        if c.name in connection_names:
            raise ValidationError(
                "A connection with name `{}` must be unique.".format(c.name)
            )
        connection_names.add(c.name)


def check_postgres(postgresql, external_services):
    postgresql_disabled = postgresql.enabled is False if postgresql else False
    external_postgresql = None
    if external_services:
        external_postgresql = external_services.postgresql

    if postgresql_disabled and not external_postgresql:
        raise ValidationError(
            "A postgresql instance is required, "
            "please enable the in-cluster postgresql, "
            "or provide an external instance."
        )


def check_rabbitmq(rabbitmq, external_services, broker):
    rabbitmq_enabled = rabbitmq and rabbitmq.enabled
    external_rabbitmq = None
    rabbitmq_borker = broker != "redis"
    if external_services:
        external_rabbitmq = external_services.rabbitmq

    if rabbitmq_enabled and external_rabbitmq:
        raise ValidationError(
            "You can either enable the in-cluster rabbitmq or use an external instance, "
            "not both!"
        )
    rabbitmq_used = rabbitmq_enabled or external_rabbitmq
    if rabbitmq_used and not rabbitmq_borker:
        raise ValidationError(
            "rabbitmq is enabled but you are using a different broker backend!"
        )

    return rabbitmq_used


def check_redis(redis, external_services, broker):
    redis_enabled = redis and redis.enabled
    redis_non_broker = redis and redis.non_broker
    external_redis = None
    redis_broker = broker == "redis"
    if external_services:
        external_redis = external_services.redis

    if redis_enabled and external_redis:
        raise ValidationError(
            "You can either enable the in-cluster redis or use an external instance, "
            "not both!"
        )

    redis_used = redis_enabled or external_redis
    if redis_used and not redis_broker and not redis_non_broker:
        raise ValidationError(
            "redis is enabled but you are using a different broker backend!"
        )

    return redis_used, not redis_non_broker


def broker_is_required(services):
    for s in services:
        if s and s.enabled:
            return True
    return False


def wrong_agent_deployment_keys(**kwargs):
    error_keys = []
    for k, v in kwargs.items():
        if v is not None:
            error_keys.append(k)
    if error_keys:
        raise ValidationError(
            "Agent deployment received some keys that are not required.\n"
            "Please remove these config keys from your config file:\n"
            "{}".format(error_keys)
        )


def validate_platform_deployment(
    postgresql, redis, rabbitmq, broker, scheduler, worker, beat, external_services
):
    check_postgres(postgresql, external_services)
    redis_used, redis_is_broker = check_redis(redis, external_services, broker)
    rabbitmq_used = check_rabbitmq(rabbitmq, external_services, broker)
    if rabbitmq_used and redis_used and redis_is_broker:
        raise ValidationError(
            "You only need to enable rabbitmq or redis for the broker, "
            "you don't need to deploy both!"
        )
    broker_defined = rabbitmq_used or redis_used
    services = [scheduler, worker, beat]
    if broker_is_required(services) and not broker_defined:
        raise ValidationError(
            "You enabled some services that require a broker, please set redis or rabbitmq!"
        )


def validate_deployment_chart(
    deployment_chart,
    agent,
    environment,
):
    if deployment_chart == DeploymentCharts.AGENT and not agent:
        raise ValidationError(
            "Agent deployment requires a valid `agent` key configuration."
        )

    if (
        deployment_chart == DeploymentCharts.PLATFORM
        and agent
        and environment != "staging"
    ):
        raise ValidationError("Platform deployment received an unexpected `agent` key.")


def validate_gateway(gateway):
    if not gateway or not gateway.service:
        return
    service_type = gateway.service.get("type")
    if service_type and service_type not in ServiceTypes.VALUES:
        raise ValidationError(
            "Received an invalid gateway service type: {}".format(service_type)
        )


class DeploymentSchema(BaseCamelSchema):
    deployment_type = fields.Str(
        allow_none=True, validate=validate.OneOf(DeploymentTypes.VALUES)
    )
    deployment_chart = fields.Str(
        allow_none=True,
        validate=validate.OneOf(DeploymentCharts.VALUES),
        default=DeploymentCharts.PLATFORM,
    )
    deployment_version = fields.Str(allow_none=True)
    release_name = fields.Str(allow_none=True)
    namespace = fields.Str(allow_none=True)
    rbac = fields.Nested(RBACSchema, allow_none=True)
    polyaxon_secret = fields.Str(allow_none=True)
    internal_token = fields.Str(allow_none=True)
    password_length = fields.Int(allow_none=True)
    ssl = fields.Nested(SSLSchema, allow_none=True)
    encryption_secret = fields.Str(allow_none=True)
    platform_secret = fields.Str(allow_none=True)
    agent_secret = fields.Str(allow_none=True)
    timezone = fields.Str(allow_none=True)
    environment = fields.Str(allow_none=True)
    ingress = fields.Nested(IngressSchema, allow_none=True)
    user = fields.Nested(RootUserSchema, allow_none=True)
    node_selector = fields.Dict(allow_none=True)
    tolerations = fields.List(fields.Dict(allow_none=True), allow_none=True)
    affinity = fields.Dict(allow_none=True)
    limit_resources = fields.Bool(allow_none=True)
    global_replicas = fields.Int(allow_none=True)
    global_concurrency = fields.Int(allow_none=True)
    gateway = fields.Nested(ApiServiceSchema, allow_none=True)
    api = fields.Nested(ApiServiceSchema, allow_none=True)
    streams = fields.Nested(ApiServiceSchema, allow_none=True)
    scheduler = fields.Nested(WorkerServiceSchema, allow_none=True)
    worker = fields.Nested(WorkerServiceSchema, allow_none=True)
    beat = fields.Nested(ServiceSchema, allow_none=True)
    agent = fields.Nested(AgentServiceSchema, allow_none=True)
    operator = fields.Nested(OperatorServiceSchema, allow_none=True)
    init = fields.Nested(HelperServiceSchema, allow_none=True)
    sidecar = fields.Nested(HelperServiceSchema, allow_none=True)
    tables_hook = fields.Nested(ServiceSchema, allow_none=True)
    clean_hooks = fields.Nested(ServiceSchema, allow_none=True)
    hooks = fields.Nested(HooksSchema, allow_none=True)
    postgresql = fields.Nested(PostgresqlSchema, allow_none=True)
    redis = fields.Nested(RedisSchema, allow_none=True)
    rabbitmq = fields.Nested(RabbitmqSchema, data_key="rabbitmq-ha", allow_none=True)
    broker = fields.Str(allow_none=True, validate=validate.OneOf(["redis", "rabbitmq"]))
    email = fields.Nested(EmailSchema, allow_none=True)
    ldap = fields.Raw(allow_none=True)
    metrics = fields.Raw(allow_none=True)
    image_pull_secrets = fields.List(fields.Str(), allow_none=True)
    host_name = fields.Str(allow_none=True)
    allowed_hosts = fields.List(fields.Str(), allow_none=True)
    include_host_ips = fields.Bool(allow_none=True)
    intervals = fields.Nested(IntervalsSchema, allow_none=True)
    artifacts_store = fields.Nested(ConnectionTypeSchema, allow_none=True)
    connections = fields.List(fields.Nested(ConnectionTypeSchema), allow_none=True)
    notification_connections = fields.List(
        fields.Nested(ConnectionTypeSchema),
        allow_none=True,
    )
    log_level = fields.Str(allow_none=True)
    security_context = fields.Nested(SecurityContextSchema, allow_none=True)
    external_services = fields.Nested(ExternalServicesSchema, allow_none=True)
    debug_mode = fields.Bool(allow_none=True)
    organization_key = fields.Str(allow_none=True)
    auth = fields.Nested(AuthSchema, allow_none=True)
    ui = fields.Nested(UISchema, allow_none=True)
    include_chart_revision = fields.Bool(allow_none=True)
    operators = fields.Nested(OperatorsSchema, allow_none=True)
    istio = fields.Dict(allow_none=True)
    # Pending validation
    dns = fields.Raw(allow_none=True)

    @staticmethod
    def schema_config():
        return DeploymentConfig

    @validates_schema
    def validate_connections(self, data, **kwargs):
        validate_connections(data.get("artifacts_store"), data.get("connections"))

    @validates_schema
    def validate_deployment(self, data, **kwargs):
        validate_deployment_chart(
            deployment_chart=data.get("deployment_chart"),
            agent=data.get("agent"),
            environment=data.get("environment"),
        )
        validate_platform_deployment(
            postgresql=data.get("postgresql"),
            redis=data.get("redis"),
            rabbitmq=data.get("rabbitmq"),
            broker=data.get("broker"),
            scheduler=data.get("scheduler"),
            worker=data.get("worker"),
            beat=data.get("beat"),
            external_services=data.get("external_services"),
        )
        validate_gateway(data.get("gateway"))
        if data.get("deployment_chart") == DeploymentCharts.AGENT:
            wrong_agent_deployment_keys(
                polyaxon_secret=data.get("polyaxon_secret"),
                internal_token=data.get("internal_token"),
                password_length=data.get("password_length"),
                user=data.get("user"),
                global_replicas=data.get("global_replicas"),
                global_concurrency=data.get("global_concurrency"),
                api=data.get("api"),
                scheduler=data.get("scheduler"),
                worker=data.get("worker"),
                beat=data.get("beat"),
                tables_hook=data.get("tables_hook"),
                hooks=data.get("hooks"),
                postgresql=data.get("postgresql"),
                redis=data.get("redis"),
                rabbitmq=data.get("rabbitmq"),
                broker=data.get("broker"),
                email=data.get("email"),
                ldap=data.get("ldap"),
                intervals=data.get("intervals"),
                metrics=data.get("metrics"),
                organization_key=data.get("organization_key"),
                ui=data.get("ui"),
            )


class DeploymentConfig(BaseConfig):
    SCHEMA = DeploymentSchema

    def __init__(
        self,
        deployment_type=None,
        deployment_chart=None,
        deployment_version=None,
        release_name=None,
        namespace=None,
        rbac=None,
        polyaxon_secret=None,
        internal_token=None,
        password_length=None,
        ssl=None,
        encryption_secret=None,
        platform_secret=None,
        agent_secret=None,
        timezone=None,
        environment=None,
        ingress=None,
        user=None,
        node_selector=None,
        tolerations=None,
        affinity=None,
        limit_resources=None,
        global_replicas=None,
        global_concurrency=None,
        gateway=None,
        api=None,
        streams=None,
        scheduler=None,
        worker=None,
        beat=None,
        agent=None,
        operator=None,
        init=None,
        sidecar=None,
        tables_hook=None,
        clean_hooks=None,
        hooks=None,
        postgresql=None,
        redis=None,
        rabbitmq=None,
        broker=None,
        email=None,
        ldap=None,
        metrics=None,
        image_pull_secrets=None,
        host_name=None,
        allowed_hosts=None,
        include_host_ips=None,
        intervals=None,
        artifacts_store=None,
        connections=None,
        notification_connections=None,
        log_level=None,
        security_context=None,
        external_services=None,
        debug_mode=None,
        auth=None,
        organization_key=None,
        dns=None,
        ui=None,
        operators=None,
        istio=None,
        include_chart_revision=None,
    ):
        validate_deployment_chart(
            deployment_chart=deployment_chart,
            agent=agent,
            environment=environment,
        )
        validate_platform_deployment(
            postgresql=postgresql,
            redis=redis,
            rabbitmq=rabbitmq,
            broker=broker,
            scheduler=scheduler,
            worker=worker,
            beat=beat,
            external_services=external_services,
        )
        validate_gateway(gateway)
        self.deployment_type = deployment_type
        self.deployment_chart = deployment_chart or DeploymentCharts.PLATFORM
        self.deployment_version = deployment_version
        self.release_name = release_name
        self.namespace = namespace
        self.rbac = rbac
        self.polyaxon_secret = polyaxon_secret
        self.internal_token = internal_token
        self.password_length = password_length
        self.ssl = ssl
        self.dns = dns
        self.encryption_secret = encryption_secret
        self.platform_secret = platform_secret
        self.agent_secret = agent_secret
        self.timezone = timezone
        self.environment = environment
        self.ingress = ingress
        self.user = user
        self.node_selector = node_selector
        self.tolerations = tolerations
        self.affinity = affinity
        self.limit_resources = limit_resources
        self.global_replicas = global_replicas
        self.global_concurrency = global_concurrency
        self.gateway = gateway
        self.api = api
        self.streams = streams
        self.scheduler = scheduler
        self.worker = worker
        self.beat = beat
        self.agent = agent
        self.operator = operator
        self.init = init
        self.sidecar = sidecar
        self.tables_hook = tables_hook
        self.clean_hooks = clean_hooks
        self.hooks = hooks
        self.postgresql = postgresql
        self.redis = redis
        self.rabbitmq = rabbitmq
        self.broker = broker
        self.email = email
        self.ldap = ldap
        self.metrics = metrics
        self.image_pull_secrets = image_pull_secrets
        self.host_name = host_name
        self.allowed_hosts = allowed_hosts
        self.include_host_ips = include_host_ips
        self.intervals = intervals
        self.artifacts_store = artifacts_store
        self.connections = connections
        self.notification_connections = notification_connections
        self.log_level = log_level
        self.security_context = security_context
        self.external_services = external_services
        self.debug_mode = debug_mode
        self.auth = auth
        self.organization_key = organization_key
        self.ui = ui
        self.operators = operators
        self.istio = istio
        self.include_chart_revision = include_chart_revision
        if self.deployment_chart == DeploymentCharts.AGENT:
            wrong_agent_deployment_keys(
                polyaxon_secret=polyaxon_secret,
                internal_token=internal_token,
                password_length=password_length,
                platform_secret=platform_secret,
                encryption_secret=encryption_secret,
                user=user,
                global_replicas=global_replicas,
                global_concurrency=global_concurrency,
                api=api,
                scheduler=scheduler,
                worker=worker,
                beat=beat,
                tables_hook=tables_hook,
                hooks=hooks,
                postgresql=postgresql,
                redis=redis,
                rabbitmq=rabbitmq,
                broker=broker,
                email=email,
                ldap=ldap,
                intervals=intervals,
                metrics=metrics,
                organization_key=organization_key,
                ui=ui,
            )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.connections.schemas import V1K8sResourceSchema
from polyaxon.env_vars.keys import (
    POLYAXON_KEYS_API_VERSION,
    POLYAXON_KEYS_AUTH_TOKEN,
    POLYAXON_KEYS_AUTHENTICATION_TYPE,
    POLYAXON_KEYS_HEADER,
    POLYAXON_KEYS_HEADER_SERVICE,
    POLYAXON_KEYS_HOST,
    POLYAXON_KEYS_IS_MANAGED,
    POLYAXON_KEYS_K8S_NAMESPACE,
    POLYAXON_KEYS_K8S_POD_ID,
    POLYAXON_KEYS_RUN_INSTANCE,
    POLYAXON_KEYS_SECRET_INTERNAL_TOKEN,
    POLYAXON_KEYS_SECRET_KEY,
)
from polyaxon.exceptions import PolypodException
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common.env_vars import (
    get_env_from_config_map,
    get_env_from_config_maps,
    get_env_from_k8s_resources,
    get_env_from_secret,
    get_env_from_secrets,
    get_env_var,
    get_env_vars_from_k8s_resources,
    get_from_config_map,
    get_from_field_ref,
    get_from_secret,
    get_items_from_config_map,
    get_items_from_secret,
    get_kv_env_vars,
    get_resources_env_vars,
    get_run_instance_env_var,
    get_service_env_vars,
    get_str_var,
)
from polyaxon.schemas.types import V1K8sResourceType
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestEnvVars(BaseTestCase):
    def test_get_str_var(self):
        # String value
        var = get_str_var(value="bar")
        assert var == "bar"
        # Int value
        var = get_str_var(value=1)
        assert var == "1"
        # Dict value
        var = get_str_var(value={"moo": "bar"})
        assert var == '{"moo": "bar"}'
        # Empty value
        var = get_str_var(value=None)
        assert var == ""
        # 0 value
        var = get_str_var(value=0)
        assert var == "0"

    def test_get_env_vars(self):
        # String value
        env_var = get_env_var(name="foo", value="bar")
        assert env_var.name == "foo"
        assert env_var.value == "bar"
        # Int value
        env_var = get_env_var(name="foo", value=1)
        assert env_var.name == "foo"
        assert env_var.value == "1"
        # Dict value
        env_var = get_env_var(name="foo", value={"moo": "bar"})
        assert env_var.name == "foo"
        assert env_var.value == '{"moo": "bar"}'

    def test_get_kv_env_var(self):
        # Empty value
        assert get_kv_env_vars([]) == []
        # Non valid value
        with self.assertRaises(PolypodException):
            get_kv_env_vars([[123, "foo", "bar"]])
        with self.assertRaises(PolypodException):
            get_kv_env_vars([[123]])
        # Valid value
        env_vars = get_kv_env_vars(
            [["foo", {"moo": "bar"}], ("foo", "bar"), ["foo", 1]]
        )
        assert env_vars[0].name == "foo"
        assert env_vars[0].value == '{"moo": "bar"}'
        assert env_vars[1].name == "foo"
        assert env_vars[1].value == "bar"
        assert env_vars[2].name == "foo"
        assert env_vars[2].value == "1"

    def test_get_resources_env_vars(self):
        env_vars = get_resources_env_vars(None)
        assert len(env_vars) == 1
        assert env_vars[0].name == "NVIDIA_VISIBLE_DEVICES"
        assert env_vars[0].value == "none"

        resources = k8s_schemas.V1ResourceRequirements(limits={"cpu": 1})
        env_vars = get_resources_env_vars(resources)
        assert len(env_vars) == 1
        assert env_vars[0].name == "NVIDIA_VISIBLE_DEVICES"
        assert env_vars[0].value == "none"

        resources = k8s_schemas.V1ResourceRequirements(limits={"memory": 1})
        env_vars = get_resources_env_vars(resources)
        assert len(env_vars) == 1
        assert env_vars[0].name == "NVIDIA_VISIBLE_DEVICES"
        assert env_vars[0].value == "none"

        resources = k8s_schemas.V1ResourceRequirements(requests={"nvidia.com/gpu": 1})
        env_vars = get_resources_env_vars(resources)
        assert len(env_vars) == 0
        assert env_vars == []

    def test_get_from_config_map(self):
        env_var = get_from_config_map(
            key_name="foo", config_map_key_name="cm_key", config_map_ref_name="cm_ref"
        )
        assert env_var.name == "foo"
        assert isinstance(env_var.value_from, k8s_schemas.V1EnvVarSource)
        assert env_var.value_from.config_map_key_ref.name == "cm_ref"
        assert env_var.value_from.config_map_key_ref.key == "cm_key"

    def test_get_from_secret(self):
        env_var = get_from_secret(
            key_name="foo", secret_key_name="secret_key", secret_ref_name="secret_ref"
        )
        assert env_var.name == "foo"
        assert isinstance(env_var.value_from, k8s_schemas.V1EnvVarSource)
        assert env_var.value_from.secret_key_ref.name == "secret_ref"
        assert env_var.value_from.secret_key_ref.key == "secret_key"

    def test_get_items_from_secret(self):
        # None
        assert get_items_from_secret(None) == []
        # Secret without items
        secret = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test"), is_requested=True
        )
        assert get_items_from_secret(secret) == []
        secret = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=[]),
            is_requested=True,
        )
        assert get_items_from_secret(secret) == []
        # Secret with items
        secret = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=["item1", "item2"]),
            is_requested=True,
        )
        assert get_items_from_secret(secret) == [
            get_from_secret("item1", "item1", secret.schema.name),
            get_from_secret("item2", "item2", secret.schema.name),
        ]

    def get_items_from_config_map(self):
        # None
        assert get_items_from_secret(None) == []
        # Secret without items
        secret = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test"), is_requested=True
        )
        assert get_items_from_secret(secret) == []
        secret = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=[]),
            is_requested=True,
        )
        assert get_items_from_secret(secret) == []
        # Secret with items
        secret = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=["item1", "item2"]),
            is_requested=True,
        )
        assert get_items_from_secret(secret) == [
            get_from_config_map("item1", "item1", secret.schema.name),
            get_from_config_map("item2", "item2", secret.schema.name),
        ]

    def test_get_env_vars_from_k8s_resources(self):
        assert get_env_vars_from_k8s_resources(secrets=[], config_maps=[]) == []
        res1 = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test"), is_requested=True
        )
        res2 = V1K8sResourceType(
            name="test2", schema=V1K8sResourceSchema(name="test2"), is_requested=True
        )
        assert (
            get_env_vars_from_k8s_resources(secrets=[res1, res2], config_maps=[]) == []
        )
        assert get_env_vars_from_k8s_resources(secrets=[res1], config_maps=[res2]) == []
        assert (
            get_env_vars_from_k8s_resources(secrets=[], config_maps=[res1, res2]) == []
        )

        res1 = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=["item1", "item2"]),
            is_requested=True,
        )
        res2 = V1K8sResourceType(
            name="test2",
            schema=V1K8sResourceSchema(name="test2", items=["item1", "item2"]),
            is_requested=True,
        )
        expected = get_items_from_secret(res1) + get_items_from_secret(res2)
        assert (
            get_env_vars_from_k8s_resources(secrets=[res1, res2], config_maps=[])
            == expected
        )
        expected = get_items_from_secret(res1) + get_items_from_config_map(res2)
        assert (
            get_env_vars_from_k8s_resources(secrets=[res1], config_maps=[res2])
            == expected
        )
        expected = get_items_from_config_map(res1) + get_items_from_config_map(res2)
        assert (
            get_env_vars_from_k8s_resources(secrets=[], config_maps=[res1, res2])
            == expected
        )

    def test_get_from_field_ref(self):
        env_var = get_from_field_ref(name="test", field_path="metadata.name")
        assert env_var.name == "test"
        assert env_var.value_from.field_ref.field_path == "metadata.name"

    def test_get_env_from_secret(self):
        # None
        assert get_env_from_secret(secret=None) is None
        # Secret with items
        secret = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=["item1", "item2"]),
            is_requested=True,
        )
        assert get_env_from_secret(secret=secret) is None

        # Secret
        secret = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test_ref"), is_requested=True
        )

        assert get_env_from_secret(secret=secret).secret_ref == {"name": "test_ref"}

    def test_get_env_from_config_map(self):
        # None
        assert get_env_from_config_map(config_map=None) is None
        # ConfigMap with items
        config_map = V1K8sResourceType(
            name="test",
            schema=V1K8sResourceSchema(name="test", items=["item1", "item2"]),
            is_requested=True,
        )
        assert get_env_from_config_map(config_map=config_map) is None

        # ConfigMap
        config_map = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test_ref"), is_requested=True
        )

        assert get_env_from_config_map(config_map=config_map).config_map_ref == {
            "name": "test_ref"
        }

    def test_get_env_from_secrets(self):
        # None
        assert get_env_from_secrets(secrets=None) == []
        # Secret with items
        secret1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(name="test1", items=["item1", "item2"]),
            is_requested=True,
        )
        # Secret
        secret2 = V1K8sResourceType(
            name="test2", schema=V1K8sResourceSchema(name="test_ref"), is_requested=True
        )

        assert get_env_from_secrets(secrets=[secret1, secret2]) == [
            get_env_from_secret(secret2)
        ]

    def test_get_env_from_config_maps(self):
        # None
        assert get_env_from_config_maps(config_maps=None) == []
        # ConfigMap with items
        config_map1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(name="test1", items=["item1", "item2"]),
            is_requested=True,
        )
        # ConfigMap
        config_map2 = V1K8sResourceType(
            name="test2", schema=V1K8sResourceSchema(name="test_ref"), is_requested=True
        )

        assert get_env_from_config_maps(config_maps=[config_map1, config_map2]) == [
            get_env_from_config_map(config_map2)
        ]

    def test_get_env_from_k8s_resources(self):
        assert get_env_from_k8s_resources(secrets=[], config_maps=[]) == []
        res1 = V1K8sResourceType(
            name="test", schema=V1K8sResourceSchema(name="test"), is_requested=True
        )
        res2 = V1K8sResourceType(
            name="test2", schema=V1K8sResourceSchema(name="test2"), is_requested=True
        )
        expected = get_env_from_secrets(secrets=[res1, res2])
        assert (
            get_env_from_k8s_resources(secrets=[res1, res2], config_maps=[]) == expected
        )
        expected = get_env_from_secrets(secrets=[res1]) + get_env_from_config_maps(
            config_maps=[res2]
        )
        assert (
            get_env_from_k8s_resources(secrets=[res1], config_maps=[res2]) == expected
        )
        expected = get_env_from_config_maps(config_maps=[res1, res2])
        assert (
            get_env_from_k8s_resources(secrets=[], config_maps=[res1, res2]) == expected
        )

    def get_run_instance_env_var(self):
        assert get_run_instance_env_var() == get_from_field_ref(
            name=POLYAXON_KEYS_RUN_INSTANCE,
            field_path="metadata.labels['run_instance']",
        )

    def test_get_service_env_vars_raises_for_internal_and_agent_token(self):
        with self.assertRaises(PolypodException):
            get_service_env_vars(
                header=None,
                service_header=None,
                include_secret_key=False,
                include_internal_token=True,
                include_agent_token=True,
                authentication_type=None,
                polyaxon_default_secret_ref="polyaxon-secret",
                polyaxon_agent_secret_ref="polyaxon-agent",
                api_host="localhost",
                api_version="v1",
                run_instance="foo.bar.runs.run_uuid",
            )

    def test_get_service_env_vars(self):
        env_vars = get_service_env_vars(
            header=None,
            service_header=None,
            include_secret_key=False,
            include_internal_token=False,
            include_agent_token=False,
            authentication_type=None,
            polyaxon_default_secret_ref="polyaxon-secret",
            polyaxon_agent_secret_ref="polyaxon-agent",
            api_host="localhost",
            api_version="v1",
            run_instance="foo.bar.runs.run_uuid",
        )
        assert len(env_vars) == 8
        # TODO: Remove in v1.2 Compatibility API_HOST
        # assert len(env_vars) == 7
        env_var_names = [env_var.name for env_var in env_vars]
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_K8S_NAMESPACE in env_var_names
        assert POLYAXON_KEYS_HOST in env_var_names
        assert POLYAXON_KEYS_IS_MANAGED in env_var_names
        assert POLYAXON_KEYS_API_VERSION in env_var_names
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_RUN_INSTANCE in env_var_names

        env_vars = get_service_env_vars(
            header="foo",
            service_header="foo",
            include_secret_key=True,
            include_internal_token=True,
            include_agent_token=False,
            authentication_type="foo",
            polyaxon_default_secret_ref="polyaxon-secret",
            polyaxon_agent_secret_ref="polyaxon-agent",
            api_host="localhost",
            api_version="v1",
            run_instance="foo.bar.runs.run_uuid",
        )
        assert len(env_vars) == 13
        # TODO: Remove in v1.2 Compatibility API_HOST
        # assert len(env_vars) == 12
        env_var_names = [env_var.name for env_var in env_vars]
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_K8S_NAMESPACE in env_var_names
        assert POLYAXON_KEYS_HOST in env_var_names
        assert POLYAXON_KEYS_IS_MANAGED in env_var_names
        assert POLYAXON_KEYS_API_VERSION in env_var_names
        assert POLYAXON_KEYS_HEADER in env_var_names
        assert POLYAXON_KEYS_HEADER_SERVICE in env_var_names
        assert POLYAXON_KEYS_SECRET_KEY in env_var_names
        assert POLYAXON_KEYS_SECRET_INTERNAL_TOKEN in env_var_names
        assert POLYAXON_KEYS_AUTHENTICATION_TYPE in env_var_names
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_RUN_INSTANCE in env_var_names

        env_vars = get_service_env_vars(
            header="foo",
            service_header="foo",
            include_secret_key=True,
            include_internal_token=False,
            include_agent_token=True,
            authentication_type="foo",
            polyaxon_default_secret_ref="polyaxon-secret",
            polyaxon_agent_secret_ref="polyaxon-agent",
            api_host="localhost",
            api_version="v1",
            run_instance="foo.bar.runs.run_uuid",
        )
        assert len(env_vars) == 13
        # TODO: Remove in v1.2 Compatibility API_HOST
        # assert len(env_vars) == 12
        env_var_names = [env_var.name for env_var in env_vars]
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_K8S_NAMESPACE in env_var_names
        assert POLYAXON_KEYS_HOST in env_var_names
        assert POLYAXON_KEYS_IS_MANAGED in env_var_names
        assert POLYAXON_KEYS_API_VERSION in env_var_names
        assert POLYAXON_KEYS_HEADER in env_var_names
        assert POLYAXON_KEYS_HEADER_SERVICE in env_var_names
        assert POLYAXON_KEYS_SECRET_KEY in env_var_names
        assert POLYAXON_KEYS_AUTH_TOKEN in env_var_names
        assert POLYAXON_KEYS_AUTHENTICATION_TYPE in env_var_names
        assert POLYAXON_KEYS_K8S_POD_ID in env_var_names
        assert POLYAXON_KEYS_RUN_INSTANCE in env_var_names



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow import ValidationError

from polyaxon.connections.schemas import V1K8sResourceSchema
from polyaxon.schemas.types import V1K8sResourceType
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.parser_mark
class TestV1K8sResourceType(BaseTestCase):
    def setUp(self):
        self.spec1 = V1K8sResourceType(name="test1", is_requested=True)
        self.spec2 = V1K8sResourceType(
            name="test2", schema=V1K8sResourceSchema(name="ref2"), is_requested=False
        )
        self.spec3 = V1K8sResourceType(
            name="test3",
            schema=V1K8sResourceSchema(
                name="ref3", items=["item45"], mount_path="/some_path"
            ),
            is_requested=False,
        )
        super().setUp()

    def test_from_model(self):
        result = V1K8sResourceType.from_model(self.spec1)
        assert result.is_requested is False
        result = V1K8sResourceType.from_model(self.spec2)
        assert result == self.spec2
        result = V1K8sResourceType.from_model(self.spec3)
        assert result == self.spec3

    def test_resource_config(self):
        config_dict = {"name": "foo"}
        config = V1K8sResourceSchema.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

        config_dict = {"name": "foo", "mount_path": 213}
        with self.assertRaises(ValidationError):
            V1K8sResourceSchema.from_dict(config_dict)

        config_dict = {"name": "foo", "items": 213}
        with self.assertRaises(ValidationError):
            V1K8sResourceSchema.from_dict(config_dict)

        config_dict = {
            "name": "foo",
            "mountPath": "/foo/path",
            "items": ["item1", "item2"],
        }
        config = V1K8sResourceSchema.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)

        config_dict = {"name": "foo", "items": ["item1", "item2"]}
        config = V1K8sResourceSchema.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from polyaxon import pkg
from polyaxon.containers.names import MAIN_JOB_CONTAINER
from polyaxon.containers.pull_policy import PullPolicy
from polyaxon.k8s import k8s_schemas
from polyaxon.k8s.k8s_schemas import V1Container
from polyaxon.schemas.types import V1ConnectionType


def get_default_cleaner_container(
    store: V1ConnectionType, run_uuid: str, run_kind: str
):
    subpath = os.path.join(store.store_path, run_uuid)

    clean_args = "polyaxon clean-artifacts {} --subpath={}".format(
        store.kind.replace("_", "-"), subpath
    )
    wait_args = "polyaxon wait --uuid={} --kind={}".format(run_uuid, run_kind)
    return V1Container(
        name=MAIN_JOB_CONTAINER,
        image="polyaxon/polyaxon-init:{}".format(pkg.VERSION),
        image_pull_policy=PullPolicy.ALWAYS.value,
        command=["/bin/bash", "-c"],
        args=["{} && {}".format(wait_args, clean_args)],
        resources=k8s_schemas.V1ResourceRequirements(
            limits={"cpu": "0.5", "memory": "100Mi"},
            requests={"cpu": "0.1", "memory": "20Mi"},
        ),
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon import types
from polyaxon.auxiliaries import get_default_tuner_container
from polyaxon.k8s.k8s_schemas import V1Container
from polyaxon.polyflow import (
    V1IO,
    V1Bayes,
    V1Component,
    V1Hyperband,
    V1Hyperopt,
    V1Matrix,
    V1Operation,
    V1Param,
    V1ParamSearch,
    V1Plugins,
    V1Termination,
    V1Tuner,
)


def get_tuner(
    name: str,
    container: V1Container,
    matrix: V1Matrix,
    search: V1ParamSearch,
    iteration: int,
    bracket_iteration: int = None,
) -> V1Operation:
    params = {
        "matrix": V1Param(value=matrix.to_dict()),
        "search": V1Param(value=search.to_dict()),
        "iteration": V1Param(value=iteration),
    }
    inputs = [
        V1IO(name="matrix", iotype=types.DICT, is_list=False, is_optional=True),
        V1IO(name="search", iotype=types.DICT, is_list=False, is_optional=True),
        V1IO(name="iteration", iotype=types.INT, is_list=False, is_optional=True),
    ]
    if bracket_iteration is not None:
        params["bracket_iteration"] = V1Param(value=bracket_iteration)
        inputs.append(
            V1IO(
                name="bracket_iteration",
                iotype=types.INT,
                is_list=False,
                is_optional=True,
            )
        )
    return V1Operation(
        params=params,
        component=V1Component(
            name=name,
            plugins=V1Plugins(
                auth=True,
                collect_logs=True,
                collect_artifacts=True,
                collect_resources=False,
                sync_statuses=False,
            ),
            inputs=inputs,
            outputs=[
                V1IO(
                    name="suggestions",
                    iotype=types.DICT,
                    is_list=True,
                    is_optional=False,
                ),
            ],
            run=V1Tuner(
                container=container,
            ),
        ),
    )


def get_bo_tuner(
    matrix: V1Bayes,
    search: V1ParamSearch,
    iteration: int,
    container: V1Container = None,
) -> V1Operation:
    iteration = matrix.create_iteration(iteration)
    container = container or get_default_tuner_container(["polyaxon", "tuner", "bayes"])
    return get_tuner(
        name="bayesian-tuner",
        container=container,
        matrix=matrix,
        search=search,
        iteration=iteration,
    )


def get_hyperband_tuner(
    matrix: V1Hyperband,
    search: V1ParamSearch,
    iteration: int,
    bracket_iteration: int,
    container: V1Container = None,
) -> V1Operation:
    matrix.set_tuning_params()
    iteration, bracket_iteration = matrix.create_iteration(iteration, bracket_iteration)
    container = container or get_default_tuner_container(
        ["polyaxon", "tuner", "hyperband"],
        bracket_iteration=bracket_iteration,
    )
    return get_tuner(
        name="hyperband-tuner",
        container=container,
        matrix=matrix,
        search=search,
        iteration=iteration,
        bracket_iteration=bracket_iteration,
    )


def get_hyperopt_tuner(
    matrix: V1Hyperopt,
    search: V1ParamSearch,
    iteration: int,
    container: V1Container = None,
) -> V1Operation:
    iteration = matrix.create_iteration(iteration)
    container = container or get_default_tuner_container(
        ["polyaxon", "tuner", "hyperopt"]
    )
    return get_tuner(
        name="hyperopt-tuner",
        container=container,
        matrix=matrix,
        search=search,
        iteration=iteration,
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.notifiers.keys import INTEGRATIONS_WEBHOOKS_SLACK
from polyaxon.notifiers.spec import NotificationSpec
from polyaxon.notifiers.webhook import WebHookNotifier
from polyaxon.utils.date_utils import to_timestamp


class SlackWebHookNotifier(WebHookNotifier):
    notification_key = V1ConnectionKind.SLACK
    name = "Slack WebHook"
    description = "Slack webhooks to send payload to Slack Incoming Webhooks."
    raise_empty_context = True
    config_key = INTEGRATIONS_WEBHOOKS_SLACK
    validate_keys = ["channel", "icon_url"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        logo_url = "https://cdn.polyaxon.com/static/v1/images/logo_small.png"
        fields = []  # Use build_field

        payload = {
            "fallback": notification.condition.type,
            "title": notification.get_title(),
            "title_link": cls.get_url(),
            "text": notification.get_details(),
            "fields": fields,
            "mrkdwn_in": ["text"],
            "footer_icon": logo_url,
            "footer": "Polyaxon",
            "color": notification.get_color(),
            "ts": to_timestamp(notification.condition.last_transition_time),
        }

        return payload

    @classmethod
    def _prepare(cls, context):
        context = super()._prepare(context)

        data = {
            "fallback": context.get("fallback"),
            "title": context.get("title"),
            "title_link": context.get("title_link"),
            "text": context.get("text"),
            "fields": context.get("fields"),
            "mrkdwn_in": context.get("mrkdwn_in"),
            "footer_icon": context.get("footer_icon"),
            "footer": context.get("footer", "Polyaxon"),
            "color": context.get("color"),
        }
        return {"attachments": [data]}

    @classmethod
    def _pre_execute_web_hook(cls, data, config):
        channel = config.get("channel")
        icon_url = config.get("channel")
        if channel:
            data["channel"] = channel

        if icon_url:
            data["icon_url"] = icon_url

        return data



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from starlette.exceptions import HTTPException

from tests.test_streams.base import get_streams_client, set_store
from tests.utils import BaseTestCase


class TestMainEndpoints(BaseTestCase):
    def setUp(self):
        super().setUp()
        set_store()
        self.client = get_streams_client()

    def test_index_404(self):
        response = self.client.get("/")
        assert response.status_code == 404

    def test_404_page(self):
        response = self.client.get("/404")
        assert response.status_code == 404

    def test_500_page(self):
        with self.assertRaises(HTTPException):
            self.client.get("/500")

    def test_health_page(self):
        response = self.client.get("/healthz")
        assert response.status_code == 200



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.polyflow import V1RunKind
from polyaxon.polyflow.operations import V1CompiledOperation
from tests.utils import BaseTestCase


@pytest.mark.sidecars_mark
class TestSidecars(BaseTestCase):
    def test_sidecars_config(self):
        config_dict = {
            "kind": "compiled_operation",
            "run": {
                "kind": V1RunKind.JOB,
                "container": {"image": "foo/bar"},
                "sidecars": [
                    {"name": "sidecar1", "args": ["/subpath1", "subpath2"]},
                    {"name": "sidecar2", "args": ["/subpath1", "subpath2"]},
                ],
            },
        }
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_light_dict() == config_dict
